------------------------------
--      Are you local?      --
------------------------------

local L = AceLibrary("AceLocale-2.2"):new("TableInspector")
local tablet = AceLibrary("Tablet-2.0")

local _G = _G
local type = _G.type
local tostring = _G.tostring
local orange = "|cffeec400"

----------------------------
--      Localization      --
----------------------------

L:RegisterTranslations("enUS", function() return {
   ["Table Tablet"] = true,
   ["<Table Name>"] = true,
   ["SHOW_STRING_COPY"] = true,
   ["Strings"] = true,
   ["Numbers"] = true,
   ["Booleans"] = true,
   ["Tables"] = true,
   ["Functions"] = true,
   ["UserData"] = true,
   ["Other"] = true,
   ["Close"] = true,
   ["Click a table to view it.  Click other for access to copy."] = true,
} end)

---------------------------------
--      AceOptions Config      --
---------------------------------

local options = {
   name = L["Table Tablet"], type='text',
   desc = L["Table Tablet"],
   get = false,
   set = "NewTable",
   input = true,
   validate = function(v) return _G[v] and type(_G[v]) == 'table' and not tablet:IsRegistered(tostring(_G[v])) end,
   usage = L["<Table Name>"],
}

---------------------------------
--      Addon Declaration      --
---------------------------------
--~ TableInspector, ti
TableInspector = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceConsole-2.0", "AceDebug-2.0")
local TableInspector = TableInspector
TableInspector.revision = tonumber(string.sub("$Revision", 12, -3))

------------------------------
--      Initialization      --
------------------------------

function TableInspector:OnInitialize()
   TableInspector:RegisterChatCommand({"/TableInspector", "/ti"}, options)
end

function TableInspector:OnEnable()
--~    code mostly from Prat: UrlCopy
   StaticPopupDialogs["SHOW_STRING_COPY"] = {  
      text = "%s",
      button1 = OKAY,
      hasEditBox = 1,
      hasWideEditBox = 1,
      showAlert = 1, 
      timeout = 0,
      whileDead = 1,

      OnShow = function()
         local editBox = _G[this:GetName().."WideEditBox"]
         editBox:SetText(_G[this:GetName().."Text"]:GetText()) 
         editBox:SetFocus()
         editBox:HighlightText(0)

         local button = _G[this:GetName().."Button1"]
         button:ClearAllPoints()
         button:SetPoint("CENTER", editBox, "CENTER", 0, -30)

         _G[this:GetName().."AlertIcon"]:Hide()
      end,
      EditBoxOnEscapePressed = function() this:GetParent():Hide() end,
      EditBoxOnEnterPressed = function() this:GetParent():Hide() end,
   }

--~ to open text:
--~ StaticPopup_Show("SHOW_STRING_COPY", "test")
end

function TableInspector:OnDisable()
   StaticPopupDialogs["SHOW_STRING_COPY"] = nil
end

------------------------------
--      Core Functions      --
------------------------------

--wrap children
--string = ff a0 ff
--function = D0 FF D0
--boolean = 00 00 7F
--number = 00 7F 7F
--table

local function ScheduleClose(name)
   TableInspector:ScheduleEvent(function() 
      tablet:Close(name)
      tablet:Unregister(name) 
   end, 0.01)
end

function TableInspector:NewTable(name, t)
   t = type(t) == 'table' and t or _G[name]
   local tstr = tostring(t)
   
   if tablet:IsRegistered(tstr) then return end
   tablet:Register(tstr,
      'data', {},
      'minWidth', 300,
      'cantAttach', true,
      'clickable', true,
      'parent', "UIParent",
      'showTitleWhenDetached', true,
      'showHintWhenDetached', true,
      'dontHook', true,
      'strata', "DIALOG",
      'children', function()
         tablet:SetTitle(name)
         local cat = tablet:AddCategory('text', tstr, 'justify', "CENTER", "showWithoutChildren", true, 
				'wrap', true, 'child_child_indentation', 20, 'child_textR', .93, 'child_textG', .77, 'child_textB', 0,
				'child_child_func', StaticPopup_Show, 'child_child_arg1', "SHOW_STRING_COPY"
			)
			catTable = cat:AddCategory('text', L["Tables"], 'child_wrap', true)
         local cats = {
				['string'] = 	cat:AddCategory('text', L["Strings"], 'child_wrap', true),
				['number'] = 	cat:AddCategory('text', L["Numbers"]),
				['boolean'] = 	cat:AddCategory('text', L["Booleans"]),
				['function'] = cat:AddCategory('text', L["Functions"]),
				['userdata'] = cat:AddCategory('text', L["UserData"]),
			}
			catOther = cat:AddCategory('text', L["Other"], 'child_wrap', true)
			for k,v in pairs(t) do
				local msg = "|cffffa0ff%q|r =|cffffffff "..((type(v) == 'string' or type(v) == 'table') and "%q" or "%s").."|r"
				msg = msg:format(tostring(k), tostring(v))
				msg = msg:gsub("(%a+:)", "|cffd0ffd0%1|cffffffff")
				if type(v) == 'table' then
					catTable:AddLine('text', msg, 'func', TableInspector.NewTable, 'arg1', TableInspector, 'arg2', tostring(k), 'arg3', v)
				else
					(cats[type(v)] or catOther):AddLine('text', msg, 'arg2', tostring(v))
				end
			end
         --ugly hack unceremoniously ripped from Nargiddley, Kemayo should be shot for this - so he delcares
         tablet:AddCategory('text', orange..L["Close"], 'justify', "CENTER", 'showWithoutChildren', true,
            'func', ScheduleClose, 'arg1', tstr
         )
         tablet:SetHint(L["Click a table to view it.  Click other for access to copy."])
      end
   )
   tablet:Open(tstr)
end
